"""Code generated by Speakeasy (https://speakeasyapi.dev). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum

class ReportInput(str, Enum):
    r"""Accounting method used to prepare the cash flow statement."""
    UNKNOWN = 'Unknown'
    INDIRECT = 'Indirect'
    DIRECT = 'Direct'
