"""Code generated by Speakeasy (https://speakeasyapi.dev). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum

class SalesOrderStatus(str, Enum):
    r"""Current state of the sales order."""
    UNKNOWN = 'Unknown'
    DRAFT = 'Draft'
    OPEN = 'Open'
    CLOSED = 'Closed'
    VOID = 'Void'
