"""Code generated by Speakeasy (https://speakeasyapi.dev). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum

class PaymentLinkType(str, Enum):
    r"""Types of payment line links, either:
    `Unknown`  
    `Unlinked` - Not used  
    `Invoice` - ID refers to the invoice  
    `CreditNote` - ID refers to the credit note  
    `Refund` - ID refers to the sibling payment  
    `Payment` - ID refers to the sibling payment  
    `PaymentOnAccount` - ID refers to the customer  
    `Other` - ID refers to the customer  
    `Manual Journal`  
    `Discount` - ID refers to the payment
    """
    UNKNOWN = 'Unknown'
    UNLINKED = 'Unlinked'
    INVOICE = 'Invoice'
    CREDIT_NOTE = 'CreditNote'
    OTHER = 'Other'
    REFUND = 'Refund'
    PAYMENT = 'Payment'
    PAYMENT_ON_ACCOUNT = 'PaymentOnAccount'
    MANUAL_JOURNAL = 'ManualJournal'
    DISCOUNT = 'Discount'
