"""Code generated by Speakeasy (https://speakeasyapi.dev). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum

class ItemType(str, Enum):
    r"""Type of the item."""
    UNKNOWN = 'Unknown'
    INVENTORY = 'Inventory'
    NON_INVENTORY = 'NonInventory'
    SERVICE = 'Service'
