"""Code generated by Speakeasy (https://speakeasyapi.dev). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum

class PaymentMethodType(str, Enum):
    r"""Method of payment."""
    UNKNOWN = 'Unknown'
    CASH = 'Cash'
    CHECK = 'Check'
    CREDIT_CARD = 'CreditCard'
    DEBIT_CARD = 'DebitCard'
    BANK_TRANSFER = 'BankTransfer'
    OTHER = 'Other'
