"""Code generated by Speakeasy (https://speakeasyapi.dev). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum

class BillPaymentLineLinkType(str, Enum):
    r"""Types of links to bill payment lines."""
    UNKNOWN = 'Unknown'
    UNLINKED = 'Unlinked'
    BILL = 'Bill'
    OTHER = 'Other'
    CREDIT_NOTE = 'CreditNote'
    BILL_PAYMENT = 'BillPayment'
    PAYMENT_ON_ACCOUNT = 'PaymentOnAccount'
    REFUND = 'Refund'
    MANUAL_JOURNAL = 'ManualJournal'
    DISCOUNT = 'Discount'
